@@ -12,7 +12,6 @@ use once_cell::sync::Lazy;
 use tempfile::tempdir_in;
 
 use crate::config::Config;
-use crate::piptools::LATEST_PIP;
 use crate::platform::{
     get_app_dir, get_canonical_py_path, get_python_bin_within, get_toolchain_python_bin,
     list_known_toolchains,
@@ -33,7 +32,7 @@ pub const SELF_PYTHON_TARGET_VERSION: PythonVersionRequest = PythonVersionReques
     suffix: None,
 };
 
-const SELF_VERSION: u64 = 22;
+const SELF_VERSION: u64 = 24;
 
 pub const SELF_REQUIREMENTS: &str = r#"
 build==1.2.1
@@ -52,7 +51,7 @@ twine==5.1.1
 unearth==0.14.0
 urllib3==2.0.7
 virtualenv==20.25.0
-ruff==0.5.4
+ruff==0.6.7
 "#;
 
 static FORCED_TO_UPDATE: AtomicBool = AtomicBool::new(false);
@@ -154,8 +153,8 @@ pub fn ensure_self_venv_with_toolchain(
         let uv_venv = uv.venv(&venv_dir, &py_bin, &version, None)?;
         // write our marker
         uv_venv.write_marker()?;
-        // update pip and our requirements
-        uv_venv.update(LATEST_PIP, SELF_REQUIREMENTS)?;
+        // update our requirements
+        uv_venv.update_requirements(SELF_REQUIREMENTS)?;
 
         // Update the shims
         let shims = app_dir.join("shims");
@@ -476,9 +475,8 @@ pub fn fetch(
     // the folder into the permanent location
     if with_build_info || !installation_has_build_info(temp_dir.path()) {
         let temp_dir = temp_dir.into_path();
-        fs::rename(&temp_dir, &target_dir).map_err(|err| {
+        fs::rename(&temp_dir, &target_dir).inspect_err(|_| {
             fs::remove_dir_all(&temp_dir).ok();
-            err
         })
 
     // otherwise move the contents of the `install` folder over.
