@@ -579,6 +579,13 @@ pub struct PyProject {
     doc: DocumentMut,
 }
 
+/// Returns an implicit table.
+fn implicit() -> Item {
+    let mut table = Table::new();
+    table.set_implicit(true);
+    Item::Table(table)
+}
+
 impl PyProject {
     /// Load a pyproject toml if explicitly given, else discover from current directory
     ///
@@ -912,8 +919,14 @@ impl PyProject {
     ) -> Result<(), Error> {
         let dependencies = match kind {
             DependencyKind::Normal => &mut self.doc["project"]["dependencies"],
-            DependencyKind::Dev => &mut self.doc["tool"]["rye"]["dev-dependencies"],
-            DependencyKind::Excluded => &mut self.doc["tool"]["rye"]["excluded-dependencies"],
+            DependencyKind::Dev => self
+                .obtain_tool_config_table()?
+                .entry("dev-dependencies")
+                .or_insert(Item::Value(Value::Array(Array::new()))),
+            DependencyKind::Excluded => self
+                .obtain_tool_config_table()?
+                .entry("excluded-dependencies")
+                .or_insert(Item::Value(Value::Array(Array::new()))),
             DependencyKind::Optional(ref section) => {
                 // add this as a proper non-inline table if it's missing
                 let table = &mut self.doc["project"]["optional-dependencies"];
@@ -943,8 +956,14 @@ impl PyProject {
     ) -> Result<Option<Requirement>, Error> {
         let dependencies = match kind {
             DependencyKind::Normal => &mut self.doc["project"]["dependencies"],
-            DependencyKind::Dev => &mut self.doc["tool"]["rye"]["dev-dependencies"],
-            DependencyKind::Excluded => &mut self.doc["tool"]["rye"]["excluded-dependencies"],
+            DependencyKind::Dev => self
+                .obtain_tool_config_table()?
+                .entry("dev-dependencies")
+                .or_insert(Item::Value(Value::Array(Array::new()))),
+            DependencyKind::Excluded => self
+                .obtain_tool_config_table()?
+                .entry("excluded-dependencies")
+                .or_insert(Item::Value(Value::Array(Array::new()))),
             DependencyKind::Optional(ref section) => {
                 &mut self.doc["project"]["optional-dependencies"][section as &str]
             }
@@ -1047,6 +1066,19 @@ impl PyProject {
         fs::write(&path, self.doc.to_string()).path_context(&path, "unable to write changes")?;
         Ok(())
     }
+
+    /// Gets or creates the [tool.rye] table in pyproject.toml
+    fn obtain_tool_config_table(&mut self) -> Result<&mut Table, Error> {
+        self.doc
+            .entry("tool")
+            .or_insert(implicit())
+            .as_table_mut()
+            .ok_or(anyhow!("[tool.rye] in pyproject.toml is malformed"))?
+            .entry("rye")
+            .or_insert(implicit())
+            .as_table_mut()
+            .ok_or(anyhow!("[tool.rye] in pyproject.toml is malformed"))
+    }
 }
 
 pub fn normalize_package_name(x: &str) -> String {
@@ -1389,17 +1421,25 @@ impl ExpandedSources {
 
     /// Attach common pip args to a command.
     pub fn add_as_pip_args(&self, cmd: &mut Command) {
-        for (url, default) in self.index_urls.iter() {
-            if *default {
-                cmd.arg("--index-url");
-            } else {
-                cmd.arg("--extra-index-url");
-            }
-            cmd.arg(&url.to_string());
+        for url in self
+            .index_urls
+            .iter()
+            .filter_map(|(url, default)| if *default { Some(url) } else { None })
+        {
+            cmd.arg("--index-url");
+            cmd.arg(url.to_string());
+        }
+        for url in self
+            .index_urls
+            .iter()
+            .filter_map(|(url, default)| if *default { None } else { Some(url) })
+        {
+            cmd.arg("--extra-index-url");
+            cmd.arg(url.to_string());
         }
         for link in &self.find_links {
             cmd.arg("--find-links");
-            cmd.arg(&link.to_string());
+            cmd.arg(link.to_string());
         }
         for host in &self.trusted_hosts {
             cmd.arg("--trusted-host");
@@ -1409,18 +1449,25 @@ impl ExpandedSources {
 
     /// Write the sources to a lockfile.
     pub fn add_to_lockfile(&self, out: &mut dyn std::io::Write) -> std::io::Result<()> {
-        for (url, default) in self.index_urls.iter() {
-            if *default {
-                writeln!(out, "--index-url {}", url)?;
-            } else {
-                writeln!(out, "--extra-index-url {}", url)?;
-            }
+        for url in self
+            .index_urls
+            .iter()
+            .filter_map(|(url, default)| if *default { Some(url) } else { None })
+        {
+            writeln!(out, "--index-url {url}")?;
+        }
+        for url in self
+            .index_urls
+            .iter()
+            .filter_map(|(url, default)| if *default { None } else { Some(url) })
+        {
+            writeln!(out, "--extra-index-url {url}")?;
         }
         for link in &self.find_links {
-            writeln!(out, "--find-links {}", link)?;
+            writeln!(out, "--find-links {link}")?;
         }
         for host in &self.trusted_hosts {
-            writeln!(out, "--trusted-host {}", host)?;
+            writeln!(out, "--trusted-host {host}")?;
         }
         Ok(())
     }
